#import <Foundation/Foundation.h>

/* This class is a "transformer" which can be used for creating a tree of
NSMutableDictionaries from a prototype tree of NSMutableDictionaries, where
the two trees have different formats; that is, different keys and sub-keys.
*/



@interface SSTreeTransformer : NSObject
{
	SEL _reformatter;
	SEL _childrenInExtractor ;
	SEL _newParentMover ;
	id _contextObject ;
}

+ (SSTreeTransformer*)treeTransformerWithReformatter:(SEL)reformatter
								 childrenInExtractor:(SEL)childrenInExtractor
									  newParentMover:(SEL)newParentMover
									   contextObject:(id)contextObject ;
/*
reformatter must be an instance method of the input object's class that
returns an instance of the output class, autoreleased, representing the
a shallow copy of the receiver in the output format, or nil if the
receiver cannot be transformed due to missing keys or other corruption.
If the receiver has children, the returned dictionary should have no
children, but shall accept children using childAppender.  reformatter
may optionally accept one argument.  Alternative prototypes:
	(id)reformat ;
	(id)reformatWithContext: ;

childrenInExtractor must be an instance method of an NSMutableDictionary,
keyed with the input format, that returns an autoreleased NSArray of the
receiver's children, or nil if no children.  Prototype:
	(NSArray*)children ;

newParentMover must be an instance method of an NSMutableDictionary,
keyed with the output format, that appends the receiver to the
children of the argument.  Prototype:
	(void)moveToNewParent:(NSMutableDictionary*)newParent ;

contextObject is an object that will be passed as an argument to the
reformatter.  If reformatter does not have an argument, contextObject
must be nil.  If reformatter does have an argument, contextObject must
not be nil.
*/

+ (SSTreeTransformer*)treeTransformerWithReformatter:(SEL)reformatter
								 childrenInExtractor:(SEL)childrenInExtractor
									  newParentMover:(SEL)newParentMover ;
/*
Same as previous method, specialized to the case of no reformatter argument */

- (id)deepTransformedCopyOf:(id)nodeIn ;
/* returns a copy which the invoking method must release */

@end
